package com.codegeist.fisheye.internal.service;

import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.ConnectException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;
import java.util.zip.GZIPInputStream;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import com.codegeist.fisheye.domain.Changeset;
import com.codegeist.fisheye.domain.Changesets;
import com.codegeist.fisheye.domain.DateConverter;
import com.codegeist.fisheye.domain.PathHistory;
import com.codegeist.fisheye.domain.PathInfo;
import com.codegeist.fisheye.domain.QueryResultData;
import com.codegeist.fisheye.domain.Revision;
import com.codegeist.fisheye.domain.RevisionKey;
import com.codegeist.fisheye.domain.Version;
import com.codegeist.fisheye.domain.query.IQueryMatchListener;
import com.codegeist.fisheye.domain.query.Query;
import com.codegeist.fisheye.domain.query.Query.ReturnValue;
import com.codegeist.fisheye.service.FisheyeException;
import com.codegeist.fisheye.service.FisheyeService;


public class FisheyeServiceImpl implements FisheyeService {

	private final String baseUrl;

	public FisheyeServiceImpl(String baseUrl) {
		if (!baseUrl.endsWith("/")) {
			this.baseUrl = baseUrl + "/";
		} else {
			this.baseUrl = baseUrl;
		}
	}
	
	public Version crucibleVersion() throws FisheyeException {
		String version = new Processor<String>(baseUrl, "api/rest/crucibleVersion", new StringHandler()).process();
		return Version.fromString(version);
	}

	public Version fisheyeVersion() throws FisheyeException {
		String version = new Processor<String>(baseUrl, "api/rest/fisheyeVersion", new StringHandler()).process();
		return Version.fromString(version);
	}

	public Changeset getChangeset(String auth, String rep, String csid) throws FisheyeException {
		return new Processor<Changeset>(baseUrl, "api/rest/getChangeset?rep=" + rep + "&csid=" + csid, new ChangesetHandler()).process();
	}

	public PathInfo[] listPaths(String auth, String rep, String path) throws FisheyeException {
		
		Processor<List<PathInfo>> processor = new Processor<List<PathInfo>>(baseUrl, "api/rest/listPaths?rep=" + rep + "&path=" + path , new ArrayHandler<PathInfo>("pathinfo", new PathInfoHandler(path)));
		
		List<PathInfo> pathList = processor.process();
		
		return pathList.toArray(new PathInfo[pathList.size()]);
	}

	public Revision getRevision(String auth, String rep, String path, String rev) throws FisheyeException {
		return new Processor<Revision>(baseUrl, "api/rest/getRevision?rep=" + rep + "&path=" + path + "&rev=" + rev, new RevisionHandler()).process();
	}

	public Changesets listChangesets(String auth, String rep, String path, Date start, Date end) throws FisheyeException {
		StringBuilder sb = new StringBuilder();
		sb.append("api/rest/listChangesets?rep=");
		sb.append(rep);
		sb.append("&path=");
		sb.append(path);
		
		if (start != null) {
			sb.append("&start=");
			sb.append(DateConverter.asString(start));
		}
		
		if (end != null) {
			sb.append("&end=");
			sb.append(DateConverter.asString(end));
		}
		
		return new Processor<Changesets>(baseUrl, sb.toString(), new ChangesetsHandler()).process();
	}

	public String[] listRepositories(String auth) throws FisheyeException {
		return new Processor<String[]>(baseUrl, "api/rest/listRepositories", new StringArrayHandler()).process();
	}
	
	public PathHistory listPathHistory(String auth, String rep, String path) throws FisheyeException {
		return new Processor<PathHistory>(baseUrl, "api/rest/listPathHistory?rep=" + rep + "&path=" + path, new PathHistoryHandler()).process();
	}

	public String[] listTagsForRevision(String auth, String rep, String path, String rev) throws FisheyeException {
		return new Processor<String[]>(baseUrl, "api/rest/listTagsForRevision?rep=" + rep + "&path=" + path + "&rev=" + rev, new StringArrayHandler()).process();
	}

	public String login(String username, String password) throws FisheyeException {
		return new Processor<String>(baseUrl, "api/rest/login?username=" + username + "&password=" + password, new StringHandler()).process();
	}

	public boolean logout(String auth) throws FisheyeException {
		return new Processor<Boolean>(baseUrl, "api/rest/logout?auth=" + auth, new BooleanHandler()).process();
	}

	public void query(String auth, String rep, Query query, IQueryMatchListener resultListener) throws FisheyeException {
		try {
			ReturnValue[] returnValues = query.getReturnValues();
			String eyeQl = query.generateEyeQl();
			if (returnValues.length > 0) {
				new Processor<QueryResultData>(baseUrl, "api/rest/query?rep=" + rep + "&query=" + URLEncoder.encode(eyeQl, "utf-8"), new QueryResultsHandler(returnValues, resultListener)).process();
			} else {
				Handler handler = new ArrayHandler<RevisionKey>("revisionkey", new RevisionKeyHandler());
				RevisionKey[] revisions = new Processor<RevisionKey[]>(baseUrl, "api/rest/query?rep=" + rep + "&query=" + URLEncoder.encode(eyeQl, "utf-8"), handler).process();
				for (RevisionKey revision : revisions) {
					try {
						resultListener.matchRevisionKey(revision);
					} catch (Exception e) {
						e.printStackTrace();
						// TODO log this
					}
				}
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
	}

	private static class Processor<T> {
		
		private final String baseUrl;
		private final String path;
		private final Handler<T> handler;
		
		public Processor(String baseUrl, String path, Handler<T> handler) {
			this.baseUrl = baseUrl;
			this.path = path;
			this.handler = handler;
			
		}
		public T process() throws FisheyeException {
			try {
				URL url = new URL(baseUrl + path);
				HttpURLConnection connection = (HttpURLConnection) url.openConnection();
				connection.setRequestProperty("Accept-Encoding", "gzip");
				
				XMLReader reader = XMLReaderFactory.createXMLReader();
				reader.setContentHandler(new RestHandler(handler));
				InputStream inputStream = connection.getInputStream();
				String encoding = connection.getHeaderField("Content-Encoding");
				if (encoding != null && "gzip".equals(encoding)) {
					inputStream = new GZIPInputStream(inputStream);
				}
				reader.parse(new InputSource(inputStream));
				connection.disconnect();
				return handler.getResult();
			} catch (SAXException e) {
				Exception causedBy = e.getException();
				if (causedBy instanceof FisheyeException) {
					throw (FisheyeException)causedBy;
				}
				
				throw new FisheyeException(FisheyeException.ExceptionType.SERVER_ERROR, "Unable to process response from server");
			} catch (UnknownHostException e) {
				throw new FisheyeException(FisheyeException.ExceptionType.CONNECT_ERROR, "Unknown host");
			} catch (ConnectException e) {
				throw new FisheyeException(FisheyeException.ExceptionType.CONNECT_ERROR, "Connection failed");
			} catch (FileNotFoundException e) {
				throw new FisheyeException(FisheyeException.ExceptionType.INTERFACE_DISABLED, "Unable to locate remote API");
			} catch (Exception e) {
				throw new FisheyeException("Server error");
			}
		}
	}
}
