package com.codegeist.fisheye.internal.ui.search;

import java.util.Date;

import org.eclipse.jface.dialogs.DialogPage;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.search.ui.ISearchPage;
import org.eclipse.search.ui.ISearchPageContainer;
import org.eclipse.search.ui.NewSearchUI;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import com.codegeist.fisheye.core.Activator;
import com.codegeist.fisheye.core.connection.FisheyeConnection;
import com.codegeist.fisheye.core.connection.IFisheyeConnectionManager;
import com.codegeist.fisheye.domain.DateConverter;
import com.codegeist.fisheye.domain.query.AuthorInRestriction;
import com.codegeist.fisheye.domain.query.AuthorRestriction;
import com.codegeist.fisheye.domain.query.BranchChangeRestriction;
import com.codegeist.fisheye.domain.query.CommentRestriction;
import com.codegeist.fisheye.domain.query.ContentMatchesRestriction;
import com.codegeist.fisheye.domain.query.DateRangeRestriction;
import com.codegeist.fisheye.domain.query.LatestOnRestriction;
import com.codegeist.fisheye.domain.query.PathRestriction;
import com.codegeist.fisheye.domain.query.Query;
import com.codegeist.fisheye.domain.query.CommentRestriction.CommentRestrictionType;
import com.codegeist.fisheye.domain.query.Query.AttributeType;
import com.codegeist.fisheye.domain.query.Query.GroupByType;
import com.codegeist.fisheye.domain.query.Query.OrderByType;
import com.codegeist.fisheye.internal.ui.IHelpContextIds;
import com.codegeist.fisheye.service.FisheyeException;

public class FisheyeSearchPage extends DialogPage implements ISearchPage {

	private ISearchPageContainer container;
	private Button head;
	private Text authors;
	private Text path;
	private Text contents;
	private Text comments;
	private Text branch;
	private Text after;
	private Text before;
	private ComboViewer connection;
	private ComboViewer groupBy;
	private Button orderByDate;
	private ComboViewer repository;

	public boolean performAction() {
		Query query = new Query();
		if (head.getSelection()) {
			query.addRestriction(new LatestOnRestriction());
		}
		
		if (contents.getText().length() > 0) {
			query.addRestriction(new ContentMatchesRestriction(contents.getText()));
		}
		
		if (comments.getText().length() > 0) {
			query.addRestriction(new CommentRestriction(comments.getText(), CommentRestrictionType.MATCHES));
		}
		
		if (branch.getText().length() > 0) {
			query.addRestriction(new BranchChangeRestriction(branch.getText()));
		}
		
		if (path.getText().length() > 0) {
			query.addRestriction(new PathRestriction(path.getText(), false));
		}
		
		if (authors.getText().length() > 0) {
			if (authors.getText().indexOf(',') == -1) {
				query.addRestriction(new AuthorRestriction(authors.getText()));
			} else {
				query.addRestriction(new AuthorInRestriction(authors.getText().split(",")));
			}
		}
		
		if (after.getText().length() > 0 || before.getText().length() > 0) {

			Date beforeDate = DateConverter.asDate(before.getText());
			Date afterDate = DateConverter.asDate(after.getText());
			
			query.addRestriction(new DateRangeRestriction(afterDate, true, beforeDate, true));
		}
		
		query.addReturnValue(new Query.ReturnValue(AttributeType.PATH));
		query.addReturnValue(new Query.ReturnValue(AttributeType.REVISION));
		query.addReturnValue(new Query.ReturnValue(AttributeType.DATE));
		query.addReturnValue(new Query.ReturnValue(AttributeType.AUTHOR));
		query.addReturnValue(new Query.ReturnValue(AttributeType.REVIEWS));
		query.addReturnValue(new Query.ReturnValue(AttributeType.LINES_ADDED));
		query.addReturnValue(new Query.ReturnValue(AttributeType.LINES_REMOVED));
		query.addReturnValue(new Query.ReturnValue(AttributeType.IS_ADDED));
		
		query.setGroupBy((GroupByType) ((IStructuredSelection)groupBy.getSelection()).getFirstElement());
		if (orderByDate.getSelection()) {
			query.setOrderBy(OrderByType.DATE);
		}
		FisheyeConnection fisheyeConnection = (FisheyeConnection) ((IStructuredSelection)connection.getSelection()).getFirstElement();
		String repository = (String) ((IStructuredSelection)this.repository.getSelection()).getFirstElement();
		
		NewSearchUI.runQueryInBackground(new FisheyeQuery(fisheyeConnection, repository, query));
		return true;
	}

	public void setContainer(ISearchPageContainer container) {
		this.container = container;
	}

	@Override
	public void setVisible(boolean visible) {
		if (visible) {
			container.setPerformActionEnabled(canSearch());
		}
	}
	
	private boolean canSearch() {
		return !connection.getSelection().isEmpty() && !repository.getSelection().isEmpty();
	}
	
	public void createControl(Composite parent) {
		GridDataFactory spanGridDataFactory = GridDataFactory.swtDefaults().span(2, 1);
		GridDataFactory grabGridDataFactory = GridDataFactory.fillDefaults().grab(true, false);
		
		Composite c = new Composite(parent, SWT.NONE);
		c.setLayout(new GridLayout(2, false));
		
		Label lblConnection = new Label(c, SWT.NONE);
		lblConnection.setText("Connection:");
		
		connection = new ComboViewer(c, SWT.BORDER | SWT.READ_ONLY);
		connection.setContentProvider(new IStructuredContentProvider() {
		
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		
			public void dispose() {
			}
		
			public Object[] getElements(Object inputElement) {
				IFisheyeConnectionManager manager = (IFisheyeConnectionManager)inputElement;
				return manager.getConnections().toArray();
			}
		});
		
		connection.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				return ((FisheyeConnection)element).getName();
			}
		});
		connection.setInput(Activator.getDefault().getConnectionManager());
		connection.addSelectionChangedListener(new ISelectionChangedListener() {
		
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) connection.getSelection();
				repository.setInput(selection.getFirstElement());
				container.setPerformActionEnabled(canSearch());
			}
		});
		grabGridDataFactory.applyTo(connection.getControl());
		
		Label lblRepository = new Label(c, SWT.NONE);
		lblRepository.setText("Repository:");
		
		repository = new ComboViewer(c, SWT.BORDER | SWT.READ_ONLY);
		repository.setContentProvider(new IStructuredContentProvider() {
		
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		
			public void dispose() {
			}
		
			public Object[] getElements(Object inputElement) {
				try {
					return ((FisheyeConnection)inputElement).getService().listRepositories(null);
				} catch (FisheyeException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				return new String[0];
			}
		});
		repository.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				return (String) element;
			}
		});
		repository.addSelectionChangedListener(new ISelectionChangedListener() {
		
			public void selectionChanged(SelectionChangedEvent event) {
				container.setPerformActionEnabled(canSearch());
			}
		});
		
		grabGridDataFactory.applyTo(repository.getControl());
		
		head = new Button(c, SWT.CHECK);
		head.setText("Only most recent versions");
		spanGridDataFactory.applyTo(head);
		
		Label lblAuthors = new Label(c, SWT.NONE);
		lblAuthors.setText("Author(s)");
		
		authors = new Text(c, SWT.MULTI | SWT.BORDER | SWT.SINGLE);
		grabGridDataFactory.applyTo(authors);
		
		Label lblPath = new Label(c, SWT.NONE);
		lblPath.setText("Path");
		path = new Text(c, SWT.BORDER);
		grabGridDataFactory.applyTo(path);
		
		Label lblContents = new Label(c, SWT.NONE);
		lblContents.setText("Contents");
		
		contents = new Text(c, SWT.BORDER);
		grabGridDataFactory.applyTo(contents);
		
		Label lblComments = new Label(c, SWT.NONE);
		lblComments.setText("Comments");
		
		comments = new Text(c, SWT.BORDER);
		grabGridDataFactory.applyTo(comments);
		
		Label lblBranch = new Label(c, SWT.NONE);
		lblBranch.setText("Branch");
		branch = new Text(c, SWT.BORDER | SWT.SINGLE);
		grabGridDataFactory.applyTo(branch);
		
		Label lblAfter = new Label(c, SWT.NONE);
		lblAfter.setText("After");
		
		after = new Text(c, SWT.BORDER);
		grabGridDataFactory.applyTo(after);
		
		Label lblBefore= new Label(c, SWT.NONE);
		lblBefore.setText("Before");
		
		before = new Text(c, SWT.BORDER);
		grabGridDataFactory.applyTo(before);
		
		orderByDate = new Button(c, SWT.CHECK);
		orderByDate.setText("Order by date");
		spanGridDataFactory.applyTo(orderByDate);
		
		Label lblGroupBy = new Label(c, SWT.NONE);
		lblGroupBy.setText("Group By:");
		
		groupBy = new ComboViewer(c, SWT.BORDER | SWT.READ_ONLY);
		
		groupBy.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				GroupByType groupByType = (GroupByType) element;
				return groupByType.getEyeQl(); // TODO use a better value
			}
		});
		
		groupBy.setContentProvider(new IStructuredContentProvider() {
		
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			}
		
			public void dispose() {
			}
		
			public Object[] getElements(Object inputElement) {
				return (GroupByType[])inputElement;
			}
		});
		
		groupBy.setInput(GroupByType.values());
		groupBy.setSelection(new StructuredSelection(GroupByType.FILE));
		grabGridDataFactory.applyTo(groupBy.getControl());
		
		PlatformUI.getWorkbench().getHelpSystem().setHelp(c, IHelpContextIds.SEARCH_PAGE);
		
		setControl(c);
	}

}
